Scriptname HypoSkyUIConfig extends SKI_ConfigBase

Quest Property HypoQuest Auto
Quest Property HypoFireQuest Auto

GlobalVariable Property HypoAffectFastTravel Auto
GlobalVariable Property HypoAffectVampires Auto
GlobalVariable Property HypoBaseBodyHeat Auto
GlobalVariable Property HypoWerewolfBodyHeat Auto
GlobalVariable Property HypoEnableAnimations Auto
GlobalVariable Property HypoEnableHealthDamage Auto
GlobalVariable Property HypoEnableSaturationReport Auto
GlobalVariable Property HypothermiaInterval Auto
GlobalVariable Property HypoFireInterval Auto

GlobalVariable Property HypoPeakSeasonalTemp Auto
GlobalVariable Property HypoPeakDailyTemp Auto
GlobalVariable Property HypoTempExteriorDefault Auto
GlobalVariable Property HypoTempInteriorDefault Auto

GlobalVariable Property HypoCoreTemp Auto
GlobalVariable Property HypoSaturation Auto
GlobalVariable Property HypothermiaVersion Auto

FormList Property HypoMasterSpellList Auto

ImagespaceModifier Property imodmod auto
ImagespaceModifier Property imodsevere auto

String[] Pages

; Toggle Option bools
Bool Restart
Bool Active
Bool AffectVampires
Bool AffectFastTravel
Bool EnableAnimations
Bool EnableSaturationReport

; Toggle Options
Int MainActiveOption
Int AffectFastTravelOption
Int AffectVampireOption
Int EnableAnimationsOption
Int EnableSaturationReportOption
Int EnableHealthDamageOption

; Slider options
Int SetIntervalOption
Int SetBaseBodyHeatOption
Int SetPeakDailyOption
Int SetPeakSeasonalOption
Int SetExteriorDefaultOption
Int SetInteriorDefaultOption

; Display only
Int DisplayVersionOption
Int DisplayCoreTempOption
Int DisplaySaturationOption

;====================================================================================

Function ApplySettings()
	
	If Active 
	    If !HypoQuest.IsRunning()
	        HypoQuest.Start()
		Debug.Trace("Hypothermia SKY UI MCM: HypoQuest started.")
	    EndIf
	Else
	    DeleteHypothermiaMod(Game.GetPlayer())
	    Debug.Trace("Hypothermia SKY UI MCM: HypoQuest deleted.")
	EndIf
	
	If AffectFastTravel
	    HypoAffectFastTravel.SetValue(1)
	Else
	    HypoAffectFastTravel.SetValue(0)
	EndIf
	
	If AffectVampires
	    HypoAffectVampires.SetValue(1)
	Else
	    HypoAffectVampires.SetValue(0)
	EndIf
	
	If EnableAnimations
	    HypoEnableAnimations.SetValue(1)
	Else
	    HypoEnableAnimations.SetValue(0)
	EndIf
	
	If EnableSaturationReport
	    HypoEnableSaturationReport.SetValue(1)
	Else
	    HypoEnableSaturationReport.SetValue(0)
	EndIf

EndFunction

;====================================================================================

Event OnConfigInit()
	Pages = New String[2]
	Pages[0] = "Config"
	Pages[1] = "Information"
	
	;Load globals
	
	If HypoQuest.IsRunning() || HypoQuest.IsStarting()
		Active = True
	Else
		Active = False
	EndIf
	
	If HypoAffectVampires.GetValue() != 0
		AffectVampires = True
	Else
		AffectVampires = False
	EndIf
	
	If HypoAffectFastTravel.GetValue() != 0
		AffectFastTravel = True
	Else
		AffectFastTravel = False
	EndIf
	
	If HypoEnableAnimations.GetValue() != 0
		EnableAnimations = True
	Else
		EnableAnimations = False
	EndIf
	
	If HypoEnableSaturationReport.GetValue() != 0
		EnableSaturationReport = True
	Else
		EnableSaturationReport = False
	EndIf

EndEvent

;====================================================================================

Event OnPageReset(String Page)
	If Page == ""
		;LoadCustomContent("Hypothermia/title.dds")
		;Return
	Else
		;UnloadCustomContent()
	EndIf
	If Page == Pages[0]
		SetCursorFillMode(LEFT_TO_RIGHT)
		AddHeaderOption("Main Options")																					; Left side
		AddHeaderOption("Temperature Options")																			; Right side
		MainActiveOption = AddToggleOption("On ", Active)																; Left side
		SetPeakSeasonalOption = AddSliderOption("Peak Seasonal Temp", HypoPeakSeasonalTemp.GetValue(), "{1}")			; Right side
		AffectVampireOption = AddToggleOption("Affects Vampires", AffectVampires)										; Left side
		SetPeakDailyOption = AddSliderOption("Peak Daily Temp", HypoPeakDailyTemp.GetValue(), "{1}")					; Right side

		AffectFastTravelOption = AddToggleOption("Affects Fast Travel", AffectFastTravel)								; Left side
		SetExteriorDefaultOption = AddSliderOption("Default Exterior Temp", HypoTempExteriorDefault.GetValue(), "{1}")	; Right side
		EnableAnimationsOption = AddToggleOption("Enable Animations", EnableAnimations)									; Left side
		SetInteriorDefaultOption = AddSliderOption("Default Interior Temp", HypoTempInteriorDefault.GetValue(), "{1}")	; Right side
		EnableSaturationReportOption = AddToggleOption("Enable Saturation Report", EnableSaturationReport)				; Left side
		AddEmptyOption()																								; Right side
		
		SetCursorFillMode(TOP_TO_BOTTOM)
		
		AddEmptyOption()
		
		AddHeaderOption("More Options")
				
		SetIntervalOption = AddSliderOption("Update Interval (sec)", HypothermiaInterval.GetValue(), "{1}")
		AddEmptyOption()
		SetBaseBodyHeatOption = AddSliderOption("Base Body Heat", HypoBaseBodyHeat.GetValue(), "{1}")
		AddEmptyOption()
		
	ElseIf Page == Pages[1]
		SetCursorFillMode(TOP_TO_BOTTOM)
		
		AddHeaderOption("Version Info")
		;AddTextOption("Hypothermia Version: ", HypothermiaVersion.GetValue())
		DisplayVersionOption = AddSliderOption("Hypothermia Version: ", HypothermiaVersion.GetValue(), "{2}")
		AddEmptyOption()
		
		AddHeaderOption("Current Temperature Stats")
		;AddTextOption("Player Temperature: ", HypoCoreTemp.GetValue())
		DisplayCoreTempOption = AddSliderOption("Player Temperature: ", HypoCoreTemp.GetValue(), "{2}")
		DisplaySaturationOption = AddSliderOption("Player Saturation: ", HypoSaturation.GetValue(), "{3}")
		AddEmptyOption()		
	EndIf
EndEvent

;====================================================================================

Event OnOptionSelect(Int Option)
	If Option == MainActiveOption
		Active = !Active
		SetToggleOptionValue(MainActiveOption, Active)
	ElseIf Option == AffectFastTravelOption
		AffectFastTravel = !AffectFastTravel
		SetToggleOptionValue(AffectFastTravelOption, AffectFastTravel)
	ElseIf Option == AffectVampireOption
		AffectVampires = !AffectVampires
		SetToggleOptionValue(AffectVampireOption, AffectVampires)
	ElseIf Option == EnableAnimationsOption
		EnableAnimations = !EnableAnimations
		SetToggleOptionValue(EnableAnimationsOption, EnableAnimations)
	ElseIf Option == EnableSaturationReportOption
		EnableSaturationReport = !EnableSaturationReport
		SetToggleOptionValue(EnableSaturationReportOption, EnableSaturationReport)
	EndIf
EndEvent

;====================================================================================

Event OnOptionSliderOpen(Int Option)
	If Option == SetIntervalOption
		SetSliderDialogStartValue(HypothermiaInterval.GetValue())
		SetSliderDialogDefaultValue(1.0)
		SetSliderDialogRange(0.1, 15.0)
		SetSliderDialogInterval(0.1)
	ElseIf Option == SetBaseBodyHeatOption
		SetSliderDialogStartValue(HypoBaseBodyHeat.GetValue())
		SetSliderDialogDefaultValue(22.0)
		SetSliderDialogRange(0, 40.0)
		SetSliderDialogInterval(1.0)
	ElseIf Option == SetPeakSeasonalOption
		SetSliderDialogStartValue(HypoPeakSeasonalTemp.GetValue())
		SetSliderDialogDefaultValue(8.0)
		SetSliderDialogRange(-10.0, 30.0)
		SetSliderDialogInterval(0.5)
	ElseIf Option == SetPeakDailyOption
		SetSliderDialogStartValue(HypoPeakDailyTemp.GetValue())
		SetSliderDialogDefaultValue(8.0)
		SetSliderDialogRange(-10.0, 30.0)
		SetSliderDialogInterval(0.5)
	ElseIf Option == SetExteriorDefaultOption
		SetSliderDialogStartValue(HypoTempExteriorDefault.GetValue())
		SetSliderDialogDefaultValue(8.0)
		SetSliderDialogRange(-10.0, 30.0)
		SetSliderDialogInterval(0.5)
	ElseIf Option == SetInteriorDefaultOption
		SetSliderDialogStartValue(HypoTempInteriorDefault.GetValue())
		SetSliderDialogDefaultValue(8.0)
		SetSliderDialogRange(-10.0, 30.0)
		SetSliderDialogInterval(0.5)
	EndIf
EndEvent

Event OnOptionSliderAccept(Int Option, Float Value)
	If Option == SetIntervalOption
		HypothermiaInterval.SetValue(Value)
		SetSliderOptionValue(SetIntervalOption, HypothermiaInterval.GetValue(), "{1}")
	ElseIf Option == SetBaseBodyHeatOption
		HypoBaseBodyHeat.SetValue(Value)
		SetSliderOptionValue(SetBaseBodyHeatOption, HypoBaseBodyHeat.GetValue(), "{1}")
	ElseIf Option == SetPeakSeasonalOption
		HypoPeakSeasonalTemp.SetValue(Value)
		SetSliderOptionValue(SetPeakSeasonalOption, HypoPeakSeasonalTemp.GetValue(), "{1}")
	ElseIf Option == SetPeakDailyOption
		HypoPeakDailyTemp.SetValue(Value)
		SetSliderOptionValue(SetPeakDailyOption, HypoPeakDailyTemp.GetValue(), "{1}")
	ElseIf Option == SetExteriorDefaultOption
		HypoTempExteriorDefault.SetValue(Value)
		SetSliderOptionValue(SetExteriorDefaultOption, HypoTempExteriorDefault.GetValue(), "{1}")
	ElseIf Option == SetInteriorDefaultOption
		HypoTempInteriorDefault.SetValue(Value)
		SetSliderOptionValue(SetInteriorDefaultOption, HypoTempInteriorDefault.GetValue(), "{1}")
	EndIf
EndEvent

;====================================================================================

Event OnOptionHighlight(Int Option)
	If Option == MainActiveOption
		SetInfoText("This turns Hypothermia and its effects on and off.")
	ElseIf Option == AffectVampireOption
		SetInfoText("Make vampires vulnerable to hypothermia.")
	ElseIf Option == AffectFastTravelOption
		SetInfoText("If enabled, Hypothermia will change fast travel options under some circumstances.")
	ElseIf Option == SetIntervalOption
		SetInfoText("Lower values offer faster updates but consume more script resources. Increase if you suspect you have too many scripts running.")
	ElseIf Option == SetBaseBodyHeatOption
		SetInfoText("Lower values are more challenging.")
	ElseIf Option == SetPeakSeasonalOption
		SetInfoText("Set highest temperature bonus for the year (Sun's Height). Lower values are more challenging.")
	ElseIf Option == SetPeakDailyOption
		SetInfoText("Set highest temperature bonus during the day (early afternoon). Lower values are more challenging.")
	ElseIf Option == SetExteriorDefaultOption
		SetInfoText("Set base exterior temperature. Lower values are more challenging.")
	ElseIf Option == SetInteriorDefaultOption
		SetInfoText("Set base interior temperature. Lower values are more challenging.")
	ElseIf Option == EnableAnimationsOption
		SetInfoText("Enables hand-warming, cough, and shiver animations.")
	ElseIf Option == EnableSaturationReportOption
		SetInfoText("Adds a magic effect to the UI that displays saturation.")
	ElseIf Option == DisplayVersionOption || Option == DisplayCoreTempOption || Option == DisplaySaturationOption
		SetInfoText("Informational only, changes will have no effect.")
	EndIf
EndEvent

Event OnConfigClose()
	ApplySettings()
EndEvent

Function DeleteHypothermiaMod (Actor akActor)
	
	HypoQuest.Stop()
	HypoFireQuest.Stop()
	
	int i = 0
	
	while ( HypoMasterSpellList.GetAt(i) != None )
		akActor.RemoveSpell(HypoMasterSpellList.GetAt(i) as Spell)
		i += 1
	endwhile

	Game.EnableFastTravel(True)
	imodmod.Remove()
	imodsevere.Remove()
	
EndFunction