Scriptname Hypothermia extends Quest conditional
{Hypothermia Script}

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Hypothermia Script
; by Nitor
; v1.5
; Mar 2014
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Version Control and Globals
float ScriptVersion = 0.33								; Actual latest script version (if it says 0.33 in-game then it didn't load correctly)
GlobalVariable Property HypothermiaVersion auto			; What is saved in the game
GlobalVariable Property HypoEnableStaminaDamage auto	
GlobalVariable Property HypoEnableHealthDamage auto		
GlobalVariable Property HypoEnableShaders auto
GlobalVariable Property HypoEnableCameraShake auto	
GlobalVariable Property HypoEnableAnimations auto
GlobalVariable Property HypoEnableSaturationReport auto
GlobalVariable Property HypoAffectFastTravel auto
GlobalVariable Property HypoAffectVampires auto
GlobalVariable Property HypoSleepPenaltyOffset auto
{How far below limit1 player temp must be to give Bonechills when sleeping}
GlobalVariable Property HypothermiaInterval auto
{Realtime update interval in seconds}
GlobalVariable Property HypoFireInterval auto
{Realtime update interval in seconds}

float Property interval = 1.0 auto
{Realtime update interval in seconds}

GlobalVariable Property HypoBaseBodyHeat auto			; For difficulty setting
GlobalVariable Property HypoWerewolfBodyHeat auto

GlobalVariable Property HypoPeakSeasonalTemp auto		; For custom settings
GlobalVariable Property HypoPeakDailyTemp auto			
GlobalVariable Property HypoTempInteriorDefault auto
GlobalVariable Property HypoTempExteriorDefault auto	

bool Property verbose = false auto
{Show Debug Notifications}

Quest Property HypoFireQuest auto				; Fire scanning quest

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Spells
Spell Property hypo0 auto
{Maintaining warm core temp spell}
Spell Property hypo1 auto
{Mild Hypothermia Spell}
Spell Property hypo2 auto
{Mod Hypothermia Spell}
Spell Property hypo3 auto
{Severe Hypothermia Spell}

Spell Property windcheck auto
{Windcheck Spell for accessing winds while in Papyrus}
Spell Property swimcheck auto
{Swimcheck Spell for accessing isPlayerSwimming while in Papyrus}

Spell Property zUITempCheck auto
{Player spell that shows translated coretemp in UI}
Spell Property zUITempTrendCheck auto
{Player spell that shows coretemp trend in UI}
Spell Property zUISaturationReport auto
{Player spell that shows saturation in the UI}

Spell Property zHypoTravelPenalty auto
{Penalty Disease for Sleeping/Fast-traveling while hypothermic}

MagicEffect Property SummonFlameAtronach auto
MagicEffect Property SummonFlameThrall auto
MagicEffect Property SummonFlameThrallPotent auto

MagicEffect Property FireCloakFFSelf auto

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Contains all spells for use by the delete function
FormList Property HypoMasterSpellList auto

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Messages
Message Property HypoMsgBonechills auto


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Keywords
Keyword Property warming auto
{Keyword for Magic - Should be "Warming"}
Keyword Property warmingfire auto
{Keyword for Physical Fire - Should be "WarmingFire"}

Keyword Property WarmClothes2 auto
{Keyword for Clothing2 - Should be "WarmClothes2"}
Keyword Property WarmClothes3 auto
{Keyword for Clothing3 - Should be "WarmClothes3"}

Keyword Property WarmExtra2 auto
Keyword Property WarmExtra3 auto
Keyword Property WarmHood auto

Keyword Property ArmorCuirass auto
Keyword Property ArmorBoots auto
Keyword Property ClothingBody auto
Keyword Property ClothingFeet auto
Keyword Property ClothingHead auto


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Applicable Races
FormList Property HypoVampireRaces auto
Race Property WerewolfBeastRace auto
Race Property ArgonianRace auto
Race Property KhajiitRace auto
Race Property KhajiitRaceVampire auto
Race Property WoodElfRaceVampire auto
Race Property HighElfRaceVampire auto
Race Property DarkElfRaceVampire auto
Race Property OrcRaceVampire auto
Race Property RedguardRaceVampire auto
Race Property BretonRaceVampire auto
Race Property ImperialRaceVampire auto
Race Property NordRaceVampire auto
Race Property ArgonianRaceVampire auto
Race[] Vampires

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Temperature clothing buffs and magic effect buffs
; -- Below insulation values given in fraction of protection (i.e. 0.20 = 20% protection)
float Property inscloth1 = 0.20 auto
{Insulation of Tier 1 Clothing}
float Property inscloth2 = 0.10 auto
{Additional insulation of Tier 2 Clothing}
float Property inscloth3 = 0.15 auto
{Additional insulation of Tier 3 Clothing}

float Property insextra2 = 0.10 auto
float Property insextra3 = 0.15 auto

float Property insboots = 0.05 auto
{Insulation of boots in percentage of protection}
float Property inshood = 0.10 auto
{Insulation of hoods in percentage of protection}

; -- Below values given in actual temperature offsets
float Property bufftorch = 3.0 auto
{Temperature offset of torches in degrees C}
float Property buffmagicwarm = 100.0 auto
{Temperature offset of "warm" magic effects in degrees C}
float Property bufffire = 50.0 auto
{Temperature offset of fire proximity effects in degrees C}

; -- Below values represent a multiplier for core body temperature 
;      (i.e. 1.5 --> a core body temp of 30 becomes a temp of 45 when conducting activity)
float Property heatgenmultsprint = 1.5 auto
{Heat generation mult while sprinting}
float Property heatgenmultrun = 1.1 auto
{Heat generation mult while running}
float Property heatgenmulthorse = 1.1 auto
{Heat generation mult while riding horse}

float Property heatgenmultshivera = 1.1	auto
{Heat generation mult while shivering mildly}
float Property heatgenmultshiverb = 1.3	auto
{Heat generation mult while shivering severely}


; -- Below values given in actual temperature offsets
float Property delaytimea = 1.0	auto
{how far below mild hypothermia threshold before mild shivering}
float Property delaytimeb = 1.0	auto
{how far below moderate hypothermia threshold before severe shivering}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Temperature debuffs
; -- Below values given in actual temperature offsets (Celsius)

float Property debuffsnow = 5.0 auto
{Snow effects in degrees C}
float Property debuffrain = 3.0 auto
{Rain effects in degrees C}
float Property buffpleasant = 5.0 auto
{Pleasant effects in degrees C}

float Property debuffswimcold = 35.0 auto
{Swim effects for cold water in degrees C}
float Property debuffswimwarm = 2.0 auto
{Swim effects for warm water in degrees C}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Temperatures and Environmental Factors
GlobalVariable Property HypoCoreTemp auto			; Player's core temperature in degrees C
GlobalVariable Property HypoCoreTempGrad auto		; Core Temp change from previous interval
float coretemp = 37.0								; Actual Celsius approx. of core body temp, ranges from 31 (dying) - 37 (norm)
float bodyheatgen = 22.0 							; coretemp - bodyheatgen = ambient temp where you would still be fine naked, so about 20 C
float bodyheatmod = 0.0

float seasonal = 10.0							; Seasonal temperature base for ambient
float ambtemp = 20.0							; Environmental/ambient Temp (location based)
float ambmod = 0.0								; Ambient modifier (for specific locations)
float effectivetemp = 20.0						; Effective Temp (after windspeed/wx/clothing/magic)
float humidity = 0.4							; Humidity (0.0 - 1.0)	(Possible future use)

GlobalVariable Property HypoSaturation auto		; Player's saturation on a scale of 0.0 - 1.0
float saturation = 0.0	 						; wet effect -- measures how "wet" the player is, which affects rate of temp loss and insulation
float saturationwetrate = 0.003					; From rain - reduced when wearing cloaks
float saturationfirerate = 0.020				; Fire dry rate
float saturationdryrate = 0.001					; Normal dry rate
float maxsat = 1.0
float minsat = 0.0


float windspeed = 0.0							; From winds script (0.0 - 1.0)

bool isPlayerSwimming = false

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Temp Limits
float Property coremax = 37.0 auto
{coretemp max}
float Property limit1 = 36.5 auto
{Mild Hypothermia Temp Threshold}
float Property limit2 = 34.0 auto
{Mod Hypothermia Temp Threshold}
float Property limit3 = 30.0 auto
{Severe Hypothermia Temp Threshold}
float Property coremin = 27.0 auto
{coretemp min}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Time of Day/Season
GlobalVariable property GameHour auto					; For time of day temp adjustment
GlobalVariable Property GameMonth auto					; For seasonal temp adjustment

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Animations
Idle Property IdleWarmHands auto
Idle Property IdleWarmArms auto
Idle Property IdleUncontrollableCough auto
Idle Property LooseFullBodyStagger auto

bool playanims = false



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Compatibility with IMCN
; HypoCoreTemp gets converted to Chesko's EP and stored here (HirelingRecognizeJenassaCOPY0000), an unused vanilla Global
; Goes into IMCN's morale stat
GlobalVariable Property reporttemp auto


Event OnInit()	
	
	; Every <HypothermiaInterval> secs of realtime
	RegisterForSingleUpdate(HypothermiaInterval.GetValue())
	
	Start()
	
	; To enable reloads
	RegisterForSleep()
	
	; Reset
	RefreshSettings()
	
	Debug.Trace("Hypothermia - OnInit: Initializing")
	
	Vampires = new Race[10]
	Vampires[0] = KhajiitRaceVampire
	Vampires[1] = WoodElfRaceVampire
	Vampires[2] = HighElfRaceVampire
	Vampires[3] = DarkElfRaceVampire
	Vampires[4] = OrcRaceVampire
	Vampires[5] = RedguardRaceVampire
	Vampires[6] = BretonRaceVampire
	Vampires[7] = ImperialRaceVampire
	Vampires[8] = NordRaceVampire
	Vampires[9] = ArgonianRaceVampire
	
EndEvent

Event OnSleepStart(float afSleepStartTime, float afDesiredSleepEndTime)
	
	float dur = afDesiredSleepEndTime - afSleepStartTime
	
	if ( dur >= 1.0 )
		;Resets quests
		HypoFireQuest.Stop()
		HypoFireQuest.Start()
		Stop()
		Start()
		
		GoToState("NORM")
		Debug.Notification("Hypothermia v" + HypothermiaVersion.GetValue() + " Reset")		
	endif
	
	if ( dur > 0.04)	; Sleeping for one game hour
		
		;Is inside
		if ( Game.GetPlayer().IsInInterior() )
			HypoCoreTemp.SetValue(coremax)
			saturation *= (0.5*dur/0.04)				; For every hour you spend sleeping, saturation is reduced by half
		
			if ( Game.GetPlayer().HasEffectKeyword(warmingfire) )
				saturation = minsat
			endif
			
		
		;Is outside
		else
		
			float sky = Weather.GetCurrentWeather().GetClassification()
		
			if ( Game.GetPlayer().HasEffectKeyword(warmingfire) )			; Player warms up
				HypoCoreTemp.SetValue(coremax)
				
				if ( sky != 2 )		; If it's not raining - completely dry
					saturation = minsat
				endif
				
			elseif ( sky != 2 )		; If it's not raining - reduce saturation by 30% per hour
				saturation *= (0.7*dur/0.04)
			endif
		
		endif
		
	endif
	
	
	
EndEvent

Event OnSleepStop(bool abInterrupted)

	;Penalty if hypothermic
	if ( HypoCoreTemp.GetValue() < limit1 - HypoSleepPenaltyOffset.GetValue() )
		HypoMsgBonechills.Show()
		Game.GetPlayer().AddSpell(zHypoTravelPenalty, false)
	endif
	
	RefreshSettings()

EndEvent

Event OnReset()
	Debug.Trace("Hypothermia: Reset")
	RefreshSettings()
EndEvent

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; MAIN UPDATE LOOP
Event OnUpdate()
		
		Actor pc = Game.GetPlayer()
		
		saturation = GetUpdateSaturation(pc)
		ambtemp = GetAmbientTemp(pc) + GetWindchillModifier (pc, saturation)
		
		effectivetemp = ambtemp + (coretemp - ambtemp) * GetInsulationFactor(pc, saturation)
		effectivetemp += CheckActorStateModifier(pc) 
		effectivetemp += GetWarmModifier(pc)
		
		AdjustCoreTemp(pc, saturation)
		ApplyDamages(pc)
		
		RegisterForSingleUpdate(HypothermiaInterval.GetValue())

EndEvent
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


float Function GetAmbientTemp(Actor akActor)
	
	float temp = 1.0
	string ws  = ""
	
	;Modify amb temp
	if ( akActor.IsInInterior() ) 
		temp = HypoTempInteriorDefault.GetValue() + ambmod						; Interior defaults to global, Location types are detected via player alias script and applied via ambmod
		
	else
		temp = HypoTempExteriorDefault.GetValue() + seasonal					; Exterior defaults to global + seasonal adjustment
		
		float altitude = akActor.Z 
		float pcx = akActor.X													; Performance
		float pcy = akActor.Y

		ws = akActor.getWorldspace().GetName()
		
		if (ws == "Tamriel")	
			
			; Check region -- Northern gradual temp drop
			if (pcy > -40000.0)	;this line runs a bit north of Riverrun and the Rift.
				temp -= 2		;originally set at 2, raised to make things a bit more brutal.
			endif
			if (pcy > 8000.0)	;this line runs north of Whiterun and Rorikstead, but south of Windhelm
				temp -= 2		;originally set at 2, raised to make things a bit more brutal
			endif
			if (pcy > 69000.0)	;this line runs through Morthal
				temp -= 3		;originally set at 2, raised to make things a bit more brutal
			endif 
		
		
			;the following portions are aimed at keeping the areas with snowcover roughly at or below freezing (very roughly)
			if (pcy > 69000.0 && pcx > -20000.0) 					; coastal region east of the Pale 
				temp -= 6 											; will stay below freezing except on sunny mid-days
			endif
			if (pcy < 69000.0 && pcy > 28000.0 && pcx > -50000.0)	; block south of the above block, and extending a bit further west to encompass the snowy area south of Morthal
				temp -= 6 											; will frequently be a bit above freezing in mid-day
			endif
			if (pcy < 28000.0 && pcy > 8000.0 && pcx > 50000.0)		; block south of the above block, but limited to area around Shearpoint and to the east, running south of Windhelm
				temp -= 5  											; will usually be above freezing in mid-day, 
			endif
			if (pcy < 8000.0 && pcy > -50000.0 && pcx > 165000.0)	; (easternmost part of eastmarch)
				temp -= 5
			endif
			if (pcy < -25000.0 && pcy > -48000.0 && pcx > -36000.0 && pcx < 8300.0) 	; mountain range east of Riverwood is inexplicably covered in snow....
				temp -= 4 																; the snow here should really melt...
			endif
			if (pcx > 106900 && pcy > -18100 && pcx < 12900 && pcy < -13600) 			; this *very* loosely tracks the Eastmarch hot springs
				temp += 10
			elseif (pcx > 110550 && pcy > -26900 && pcx <= 120900 && pcy <= -13600) 	; fits right under the above swath, but does not extend as far to the west.
				temp += 10
			elseif (pcx > 120900 && pcy > -34000 && pcx < 155500 && pcy < 0) 			; covers the majority of Eastmarch
				temp += 7
			endif
			
			
			; Check altitude, only if around the level of Lake Ilinalta or higher.  
			; With very few exceptions, these altitudes exist only in southern Falkreath, the Throat and the Reach. 
			
			if (pcy < -47000 && pcx > 71000) ; This makes the effective altitude of the Rift much lower, to prevent it from dropping to freezing.
				altitude -= 140000
			endif
			
			if (altitude < 0) ; this means that altitude isn't a factor for anything lower than Lake Ilinalta
				altitude = 0
			endif
			
			temp -= (altitude/2500)		; Every 2500 units above Lake Ilinalta drops 1C. (Top of Solstheim is 14000)
		
		else
		
		endif
	
	
		temp += GetHourlyTemp(GameHour.GetValue())		; Modify for time of day only if exterior
		
		Weather wx = Weather.GetCurrentWeather()		; Consider weather only if exterior
		
		if (wx.GetClassification() == 3)
			temp -= debuffsnow
		elseif (wx.GetClassification() == 2)
			temp -= debuffrain
		elseif (wx.GetClassification() == 1)
			;nochange
		elseif (wx.GetClassification() == 0)
			temp += buffpleasant
		endif
	
	endif	

	
	return temp
	
EndFunction

float Function GetUpdateSaturation(Actor akActor)
	
	if ( akActor.GetRace() == ArgonianRace )							; Argonians unaffected by saturation
		return 0.0
	endif
	
	if (akActor.IsSwimming())	 																;if player is swimming, saturation instantly reaches 100%
		saturation = maxsat
	elseif( !akActor.IsInInterior() && Weather.GetCurrentWeather().GetClassification() == 2 )	;if it is raining, saturation gradually increases. I think this comes out to about 3% or 4% saturation every minute in the rain
		saturation += saturationwetrate * interval
	elseif ( akActor.HasEffectKeyword(warmingfire) )	 				
		saturation -= saturationfirerate * interval						;if player is by fire, dry quickly
	else	 															
		saturation -= saturationdryrate * interval						;otherwise, dry gradually
	endif

	
	if saturation > maxsat 												;saturate saturation -- hehe, compsci jokes
		saturation = maxsat
	elseif saturation < minsat
		saturation = minsat
	endif
	
	HypoSaturation.SetValue(saturation)									;report to global
	
	if (HypoEnableSaturationReport.GetValue() > 0)
		zUISaturationReport.SetNthEffectMagnitude(0,saturation*100)		;report to UI
		akActor.RemoveSpell(zUISaturationReport)						;reapply
		akActor.AddSpell(zUISaturationReport, false)
	endif
	
	return saturation

EndFunction

float Function GetWindchillModifier (Actor akActor, float saturationfactor)
	
	float windchillmod = 0
	
	; INSULATION should probably factor into this somehow
	
	; Account for windchill - This is a ridiculous simplification of windchill in the interest of simplicity (100*.20) => reduce temp by 1/5 windspeed 
	;							(e.g. 60 knot winds => -10, 30 knot knot wind => 5 degree drop)
	; Saturated clothing => windchill x2 at 100% saturation
	; Note windspeed is 60 knots+ in the College of Winterhold Interiors 100% of the time - bypassing winds via location check for playability
	
	if ( akActor.IsInInterior() )
		windchillmod = 0.0
	else
		windchillmod = -20*windspeed*(1.0 + 1.0*saturationfactor)
	endif

	return windchillmod
EndFunction

float Function CheckActorStateModifier (Actor akActor)
	
	float modtemp = 0.0
	string ws = akActor.getWorldspace().GetName()
	
	; Warmswimline defines the area south of which is warmer water (3000.0 = Just north of Whiterun)
	float warmswimline = 3000.0

	; Swimming
	if (akActor.IsSwimming())
		
		if (!akActor.IsInInterior())
			if (akActor.X > 106900 && akActor.Y > -18100 && akActor.X < 12900 && AkActor.Y < -13600) 		; covers westernmost pool in Eastmarch, and goes in a bit deeper
				modtemp = 30
			elseif (akActor.X > 110550 && akActor.Y > -26900 && akActor.X <= 120900 && akActor.Y <= -13600) ; fits right under the above swath, but does not extend as far to the west.
				modtemp = 30
			elseif (akActor.X > 120900 && akActor.Y > -34000 && akActor.X < 155500 && akActor.Y < 0) 		; covers the majority of Eastmarch
				modtemp = 30
			elseif (akActor.Y < warmswimline)
				modtemp -= debuffswimwarm					;Check for swimming in warm waters
			else
				modtemp -= debuffswimcold					;Default to swimming in cold waters
			endif
		
		else
			modtemp -= debuffswimwarm						; interior water will be a bit warmer
		
		endif
		
	endif
	
	return modtemp
	
EndFunction

float Function GetInsulationFactor (Actor akActor, float sat)

	float ins = 0.0

	;Calculate clothing layers
	if ( akActor.WornHasKeyword(ArmorCuirass) || akActor.WornHasKeyword(ClothingBody) )
		ins += inscloth1
	endif
	
	;Check for extra warmth keywords
	if ( akActor.HasEffectKeyword(WarmClothes2) )
		ins += inscloth2
	elseif ( akActor.HasEffectKeyword(WarmClothes3) )
		ins += inscloth3
	endif
	
	;Check for extras, i.e. cloaks
	if ( akActor.HasEffectKeyword(WarmExtra2) )
		ins += insextra2
	elseif ( akActor.HasEffectKeyword(WarmExtra3) )
		ins += insextra3
	endif
	
	;Check for boots
	if ( akActor.WornHasKeyword(ArmorBoots) || akActor.WornHasKeyword(ClothingFeet) )
		ins += insboots
	endif
	
	;Check for hood
	if ( akActor.WornHasKeyword(ClothingHead) || akActor.HasEffectKeyword(WarmHood) )
		ins += inshood
	endif
	
	if ( akActor.GetRace() == KhajiitRace )				; Khajiit Insulation Bonus
		ins += 0.10
	endif

	return ins * (1.0 - 0.9*sat)						; 100% Saturation takes 90% of insulation away
	
EndFunction


float Function GetWarmModifier (Actor akActor)

	float warmmod = 0.0

	;Has active warming effects
	if ( akActor.HasEffectKeyword(warming) || akActor.HasMagicEffect(FireCloakFFSelf) )
		warmmod += buffmagicwarm
	endif
	
	;Has Flame Thrall/Atronach
	if ( akActor.HasMagicEffect(SummonFlameAtronach) || akActor.HasMagicEffect(SummonFlameThrall) || akActor.HasMagicEffect(SummonFlameThrallPotent))
		warmmod += bufffire
	endif
	
	;Is near fire
	if ( akActor.HasEffectKeyword(warmingfire) )
		warmmod += bufffire
	endif
	
	;Check for equipped torch
	if (akActor.GetEquippedItemType(0) == 11)
		warmmod += bufftorch
	endif
	
	return warmmod

EndFunction

float Function GetBodyHeat(Actor akActor)

	float bodyheat = HypoBaseBodyHeat.GetValue()
	Race akRace = akActor.GetRace()
	
	;Reset body heat based on race
	
	if ( akRace == WerewolfBeastRace )
		bodyheat = HypoWerewolfBodyHeat.GetValue()
	elseif ( akRace == ArgonianRace )			; Coldblooded
		bodyheat -= 2
	elseif ( akRace == KhajiitRace )			; Not acclimated to Skyrim's cold climate
		bodyheat -= 1
	endif
	
	;Is Sprinting/Running
	if ( akActor.IsSprinting() )
		bodyheat *= heatgenmultsprint
	elseif ( akActor.IsRunning() )
		bodyheat *= heatgenmultrun
	elseif ( akActor.IsSwimming() )
		bodyheat *= heatgenmultrun
	else
		;Is shivering -- the point of this is to simulate the body's ability to generate extra heat in cold emergencies (by shivering)
		if ((coretemp < (limit1 - delaytimea)) && (coretemp > (limit2 - delaytimeb))) ; slight shivering that can be voluntarily stopped.
				bodyheat *= heatgenmultshivera
		endif
		if ((coretemp < (limit2 - delaytimeb)) && coretemp > limit3)		;uncontrollable shivering -- also, body "gives up" and stops shivering when severe hypothermia sets in.
				bodyheat *= heatgenmultshiverb
		endif 
	endif
	
	;Check for riding horse
	if (akActor.IsOnMount())
		bodyheat *= heatgenmulthorse
	endif
	
	;Apply cold resist (Frost resist ups bodyheatgen by up to 40%)
	bodyheat *= (1 + akActor.GetAV("FrostResist")/250.0 )
	
	return bodyheat + bodyheatmod
	
EndFunction


Function AdjustCoreTemp(Actor akActor, float saturationfactor)
	
	Race akRace = akActor.GetRace()
	float previoustemp
	
	if (HypoAffectVampires.GetValue() == 0 && Vampires.Find(akRace) >= 0)		; Call non-participating Vampires cold
		coretemp = limit1
		previoustemp = coretemp
	
	elseif (akActor.IsGhost())
		coretemp = HypoCoreTemp.GetValue()										; Stops adjusting coretemp if player is currently a ghost (ethereal shout)
		
	else
		coretemp = HypoCoreTemp.GetValue()
	
		;Keep track of immediate trend
		previoustemp = coretemp

		;Bodyheatgen cancels out a certain number of degrees difference
		float tempdiff = GetBodyHeat(akActor) + (effectivetemp - coretemp)
		
		if (tempdiff < 0)											;Frost resistance slows the RATE at which coretemp drops
			tempdiff *= ( 1 - akActor.GetAV("FrostResist")/100.0 )	;Apply Frost resistance only when tempdiff is reducing coretemp
		endif
		
		float thermalconduct = (0.0015 + (0.0085*(saturationfactor)))	 	;  Conductance factor - increases when you're wet, up to maximum of 7x original
		
		;Mod coretemp based on interval and tempdiff, thermalconduct is a result of saturation and tweaking/balancing
		coretemp += tempdiff*interval*thermalconduct
		
		;Correct coretemp within limits
		if (coretemp > coremax)
			coretemp = coremax
		endif
		if (coretemp < coremin)
			coretemp = coremin	
		endif
		
	endif
	
	;Report coretemp
	HypoCoreTemp.SetValue(coretemp)
	
	;Report coretempgrad
	HypoCoreTempGrad.SetValue(coretemp - previoustemp)
	
	;Report coretemp to vanilla global for compatibility
	; 120 is Frostfall's Max EP
	reporttemp.SetValueInt( ((coretemp - coremin)/(coremax - coremin) * 120.0) as int )
	

EndFunction
	
Function ApplyDamages(Actor akActor)
	
	float shake = 1 - (coretemp - coremin)/(limit2 - coremin)
	;Debug.Trace("Hypothermia: Shake = " + shake)
	
	float affectfasttravel = HypoAffectFastTravel.GetValue()
	
	Race akRace = akActor.GetRace()
	
	;Apply magic effects based on player coretemp
	
	if (akActor.IsGhost())															; Account for ghost mode
		akActor.RemoveSpell(hypo3)
		akActor.RemoveSpell(hypo2)
		akActor.RemoveSpell(hypo1)
		
	elseif (HypoAffectVampires.GetValue() == 0 && Vampires.Find(akRace) >= 0)		; Account for Vampires
		akActor.RemoveSpell(hypo3)
		akActor.RemoveSpell(hypo2)
		akActor.RemoveSpell(hypo1)
		
		if (affectfasttravel > 0)
			Game.EnableFastTravel(true)
		endif
		
	elseif (coretemp < limit3)
	
		akActor.AddSpell(hypo3, verbose)
		akActor.AddSpell(hypo2, verbose)
		akActor.AddSpell(hypo1, verbose)
		akActor.RemoveSpell(hypo0)
		
		if (affectfasttravel > 0)
			Game.EnableFastTravel(false)
		endif
		
		if (GetState() != "SEVERE")
			GoToState("SEVERE")
			Debug.Trace("Hypothermia: You now have severe hypothermia.")
		endif
		
		if (HypoEnableCameraShake.GetValue() > 0.0)
			Game.ShakeCamera(None, shake, interval*Utility.RandomFloat(afMin = 0.2) )
		endif
		
	elseif (coretemp < limit2)
		
		akActor.RemoveSpell(hypo3)
		akActor.AddSpell(hypo2, verbose)
		akActor.AddSpell(hypo1, verbose)
		akActor.RemoveSpell(hypo0)
		
		if (affectfasttravel > 0)
			Game.EnableFastTravel(false)
		endif
		
		if (GetState() != "MOD")
			GoToState("MOD")
			Debug.Trace("Hypothermia: You now have moderate hypothermia.")
		endif
		
		if (HypoEnableCameraShake.GetValue() > 0.0)
			Game.ShakeCamera(None, shake, interval*Utility.RandomFloat(afMin = 0.2) )
		endif
		
	elseif (coretemp < limit1)
	
		akActor.RemoveSpell(hypo3)
		akActor.RemoveSpell(hypo2)
		akActor.AddSpell(hypo1, verbose)
		akActor.RemoveSpell(hypo0)
		
		if (affectfasttravel > 0)
			if (HypoCoreTempGrad.GetValue() >= 0.0)
				Game.EnableFastTravel(true)
			else
				Game.EnableFastTravel(false)
			endif
		endif
		
		if (GetState() != "MILD")
			GoToState("MILD")
			Debug.Trace("Hypothermia: You now have mild hypothermia.")
		endif
		
	else
		akActor.RemoveSpell(hypo3)
		akActor.RemoveSpell(hypo2)
		akActor.RemoveSpell(hypo1)
		akActor.AddSpell(hypo0, verbose)
		
		if (affectfasttravel)
			Game.EnableFastTravel(true)
		endif
		
		GoToState("NORM")
		;Debug.Notification("You do not have hypothermia.")
		
	endif
	
EndFunction

State NORM

EndState

State MILD

	Event OnBeginState ()
		if ( playanims && HypoCoreTempGrad.GetValue() < 0.0 )
		
			;Debug.Trace("Hypo: Mild - playanims - Start Disable Controls")
			Game.DisablePlayerControls(false, false, true, false, false,false, false, false, 0)
			Game.GetPlayer().PlayIdle(IdleWarmArms)
			Utility.Wait(9)			
			Game.EnablePlayerControls()
			;Debug.Trace("Hypo: playanims - Renabled Controls")
		endif
		
	EndEvent

	Event OnEndState ()
		
	EndEvent

EndState

State MOD

	Event OnBeginState ()
		if ( playanims && HypoCoreTempGrad.GetValue() < 0.0 )
			;Debug.Trace("Hypo: Mod - playanims - Start Disable Controls")
			Game.DisablePlayerControls(true, false, true, false, false,false, false, false, 0)
			Game.GetPlayer().PlayIdle(IdleUncontrollableCough)
			Utility.Wait(3)
			Game.EnablePlayerControls()
			;Debug.Trace("Hypo: Mod - playanims - Renabled Controls")
		endif
		
	EndEvent

	Event OnEndState ()
		
	EndEvent
	
EndState

State SEVERE

	Event OnBeginState ()
		if ( playanims && HypoCoreTempGrad.GetValue() < 0.0 )
			;Debug.Trace("Hypo: Severe - playanims - Start Disable Controls")
			Game.DisablePlayerControls(true, false, true, false, false,false, false, false, 0)
			Game.GetPlayer().PlayIdle(IdleUncontrollableCough)
			Utility.Wait(3)
			Game.EnablePlayerControls()
			;Debug.Trace("Hypo: Severe - playanims - Renabled Controls")
		endif
		
	EndEvent
	
	Event OnEndState ()
		
	EndEvent
	
EndState

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Interface functions

Function SetSwimming (bool newValue)
	isPlayerSwimming = newValue
EndFunction

Function SetWindspeed (float newValue)
	windspeed = newValue
EndFunction

float Function GetWindspeed ()
	return windspeed
EndFunction

float Function GetMaxSat ()
	return maxsat
EndFunction

float Function GetSaturation ()
	return saturation
EndFunction

Function SetSaturation (float newValue)
	saturation = newValue
EndFunction

float Function SetSaturationWetRate (float newrate)
	saturationwetrate = newrate
EndFunction

float Function GetBodyHeatMod ()
	return bodyheatmod
EndFunction

Function SetBodyHeatMod (float newValue)
	bodyheatmod = newValue
EndFunction

float Function GetEffectiveTemp (Actor akActor)
	
	if (akActor == Game.GetPlayer())
		return effectivetemp
	else
		return 0.0
	endif
	
EndFunction

Function ModAmbientTemp (float modifier)
	ambmod = modifier
EndFunction

Function EnableAnims (bool newValue)
	playanims = newValue
EndFunction

float Function GetSeasonalTemp (float month)
	return -1.6 * math.abs(month - 6.0) + HypoPeakSeasonalTemp.GetValue()
EndFunction

float Function GetHourlyTemp (float hour)

	if (hour < 4)
		return -1.0
	else
		return -0.9 * math.abs(hour - 14.5) + HypoPeakDailyTemp.GetValue()
	endif
	
EndFunction


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Housekeeping
Function RefreshSettings ()

	HypothermiaVersion.SetValue(1.50)
	ScriptVersion = HypothermiaVersion.GetValue()
	interval = HypothermiaInterval.GetValue()
	
	bodyheatgen = HypoBaseBodyHeat.GetValue()
	seasonal = GetSeasonalTemp( GameMonth.GetValue() )
	
	if (HypoAffectFastTravel.GetValue() <= 0)
		Game.EnableFastTravel(true)
	endif
	
	
	; Temperature buffs and magic effect buffs
	inscloth1 = 0.20
	inscloth2 = 0.10
	inscloth3 = 0.15
	insboots = 0.05
	inshood = 0.10
	
	bufftorch = 3.0
	buffmagicwarm = 100.0 
	bufffire = 50.0
	
	heatgenmultsprint = 1.5
	heatgenmultrun = 1.1 
	heatgenmulthorse = 1.1 
	
	; Temperature debuffs
	debuffsnow = 5.0
	debuffrain = 3.0
	debuffswimcold = 35.0
	debuffswimwarm = 2.0


	; Temperatures
	ambtemp = 20.0
	effectivetemp = 20.0

	; Temp Limits
	coremax = 37.0
	limit1 = 36.5
	limit2 = 34.0
	limit3 = 30.0
	coremin = 27.0

	
	if ( Game.GetPlayer().GetRace() == KhajiitRace )				; Khajiit Insulation Bonus
		maxsat = 1.2
	else
		maxsat = 1.0
	endif
	
	Game.GetPlayer().RemoveSpell(swimcheck)
	Game.GetPlayer().RemoveSpell(windcheck)
	Game.GetPlayer().RemoveSpell(zUITempCheck)
	Game.GetPlayer().RemoveSpell(zUITempTrendCheck)
	Game.GetPlayer().RemoveSpell(zUISaturationReport)
	
	; Apply swimcheck hack
	if ( !Game.GetPlayer().AddSpell(swimcheck, verbose) )
		Debug.Trace("Hypothermia: Applying swimcheck spell failed: swimcheck will be unavailable unless PC already had spell")
	endif
	
	; Apply windcheck hack
	if ( !Game.GetPlayer().AddSpell(windcheck, verbose) )
		Debug.Trace("Hypothermia: Applying windcheck spell failed: windspeed will be unavailable unless PC already had spell")
	endif
	
	; Apply tempcheck hacks
	if ( !Game.GetPlayer().AddSpell(zUITempCheck, verbose) )
		Debug.Trace("Hypothermia: Applying tempcheck spell failed: tempcheck will be unavailable unless PC already had spell")
	endif
	
	if ( !Game.GetPlayer().AddSpell(zUITempTrendCheck, verbose) )
		Debug.Trace("Hypothermia: Applying temptrendcheck spell failed: temptrendcheck will be unavailable unless PC already had spell")
	endif
	
	; Apply saturation check
	if (HypoEnableSaturationReport.GetValue() > 0)
		if ( !Game.GetPlayer().AddSpell(zUISaturationReport, verbose) )
			Debug.Trace("Hypothermia: Applying SaturationReport spell failed: SaturationReport will be unavailable unless PC already had spell")
		endif
	endif
	
	HypoFireQuest.RegisterForSingleUpdate(HypoFireInterval.GetValue())
	RegisterForSingleUpdate(HypothermiaInterval.GetValue())
			
EndFunction

